unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  ComCtrls, StdCtrls, baseunix, unix, unixtype, unixutil, inotify;


type

  { TDirectoryEntry }

  TDirectoryEntry = Class(TCollectionItem)
  private
    FAttributes: Integer;
    FGroup: gid_t;
    FMode: mode_t;
    FName: String;
    FOwner: uid_t;
    FSize: Int64;
    FTimeStamp: TDateTime;
  Published
    Property Name : String Read FName Write FName;
    Property TimeStamp : TDateTime Read FTimeStamp Write FTimeStamp;
    Property Size : Int64 Read FSize Write FSize;
    Property Attributes : Integer Read FAttributes Write FAttributes;
    Property Mode : mode_t Read FMode Write FMode;
    Property Owner : uid_t Read FOwner Write FOwner;
    Property Group : gid_t Read FGroup Write FGroup;
  end;

  { TDirectoryEntries }

  TDirectoryEntries = Class(TCollection)
  private
    function GetE(AIndex : Integer): TDirectoryEntry;
    procedure SetE(AIndex : Integer; AValue: TDirectoryEntry);
  Public
    Function IndexOfEntry(Const AName : String) : Integer;
    Function EntryByName(Const AName : String) : TDirectoryEntry;
    Function AddEntry(Const AName : String) : TDirectoryEntry;
    Property Entries[AIndex : Integer] : TDirectoryEntry Read GetE Write SetE; default;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    LVDir: TListView;
    MDir: TMenuItem;
    MLog: TMemo;
    MIPortable: TMenuItem;
    MIOpen: TMenuItem;
    MenuItem2: TMenuItem;
    MQuit: TMenuItem;
    MMAin: TMainMenu;
    ODir: TSelectDirectoryDialog;
    procedure CheckDirChanges(Sender: TObject; var Done: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure MIPortableClick(Sender: TObject);
    procedure MIOpenClick(Sender: TObject);
    procedure MQuitClick(Sender: TObject);
  private
    FPortable : Boolean;
    FDirectory : String;
    FEntries : TDirectoryEntries;
    FD : cint;
    WD : cint;
    procedure UpDateDirectoryEntry(E: TDirectoryEntry);
    procedure UpdateDirectoryEntry(E : TDirectoryEntry; const Info: TSearchRec);
    procedure UpdateDirectoryEntry(E : TDirectoryEntry; const Info: Stat);
    procedure AddDirectoryEntry(const Info: TSearchRec);
    procedure AddDirectoryEntry(const AFileName: string; Info: Stat);
    procedure OpenDirectory(const ADirectory: string);
    procedure OpenPortableDirectory(const ADirectory: string);
    procedure OpenUnixDirectory(const ADirectory: string);
    procedure ShowDirectory;
    procedure UpdateItem(LI: TListItem);
    Procedure HandleInotifyEvent(E : Pinotify_event; Const FN : String);
    { private declarations }
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

uses fsutils;

Const
  NotifyEvents = IN_MODIFY or IN_ATTRIB or IN_CREATE or IN_DELETE;

{ TDirectoryEntries }

function TDirectoryEntries.GetE(AIndex : Integer): TDirectoryEntry;
begin
  Result:=TDirectoryEntry(Items[AIndex]);
end;

procedure TDirectoryEntries.SetE(AIndex : Integer; AValue: TDirectoryEntry);
begin
  Items[AIndex]:=AValue;
end;

function TDirectoryEntries.IndexOfEntry(const AName: String): Integer;

begin
  Result:=Count-1;
  While (Result>=0) and (GetE(Result).Name<>AName) do
    Dec(Result);
end;

function TDirectoryEntries.EntryByName(const AName: String): TDirectoryEntry;

Var
  I : Integer;

begin
  I:=IndexOfEntry(AName);
  If (I=-1) then
    Result:=Nil
  else
    Result:=GetE(I);
end;

function TDirectoryEntries.AddEntry(Const AName: String): TDirectoryEntry;
begin
  Result:=Add as TDirectoryEntry;
  Result.Name:=AName;
end;

{$R *.lfm}

{ TForm1 }

procedure TForm1.MIOpenClick(Sender: TObject);
begin
  If ODir.Execute then
    OpenDirectory(ODir.FileName);
end;

procedure TForm1.MQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  fd:=inotify_init1(0);
  wd:=-1;
  FEntries:=TDirectoryEntries.Create(TDirectoryEntry);
  FPortable:=False;
  Application.OnIdle:=@CheckDirChanges;
  OpenDirectory(ExtractFileDir(ParamStr(0)));
end;

procedure TForm1.CheckDirChanges(Sender: TObject; var Done: Boolean);

Var
  fnl,len : cint;
  e : ^inotify_event;
  buf : Array[0..1023*4] of Byte; // 4K Buffer
  fn : string;
  p : pchar;
  fds : tfdset;
  Timeout : ttimeval;
  o,awd,am : cint;

begin
  Done:=true;
  fpFD_Zero(fds);
  fpFD_SET(fd,fds);
  timeout.tv_sec:=0;
  timeout.tv_usec:=10;
  if (fpSelect(fd+1,@fds,nil,nil,@Timeout)<=0) then
    exit;
  len:=fpRead(fd,buf,sizeof(buf));
  e:=@buf;
  While ((pchar(e)-@buf)<len) do
    begin
    o:=pchar(e)-@buf;
    awd:=e^.wd;
    am:=e^.mask;
    fnl:=e^.len;
    if (fnl>0) then
      begin
      p:=@e^.name+fnl-1;
      While (p^=#0) do
        begin
        dec(p);
        dec(fnl);
        end;
      end;
    setlength(fn,fnl);
    if (fnl>0) then
      move(e^.name,fn[1],fnl);
    HandleInotifyEvent(e,fn);
    ptrint(e):=ptrint(e)+sizeof(inotify_event)+e^.len-1;
    end;
end;

procedure TForm1.HandleInotifyEvent(E : Pinotify_event; Const FN : String);

Var
  Msg : string;
  I : Integer;
  DE : TDirectoryEntry;

begin
  Msg:=Format('Change %d (%s) for file "%s".',
              [e^.mask,
               InotifyEventsToString(e^.mask),
               fn]);
  MLog.Lines.Add(Msg);
  if (fn<>'') then
    begin
    I:=Fentries.IndexOfEntry(FN);
    If (I=-1) then
      begin
      if (E^.Mask and IN_CREATE)=IN_CREATE then
        UpdateDirectoryEntry(FEntries.AddEntry(fn));
      end
    else
      if (E^.Mask and IN_DELETE)=IN_DELETE Then
        FEntries.Delete(I)
      else
        UpDateDirectoryEntry(FEntries[i]);
    end;
  ShowDirectory;
end;

procedure TForm1.UpDateDirectoryEntry(E : TDirectoryEntry);

Var
  Uinfo : TStat;
  PInfo : TSearchRec;
  FN : string;
begin
  FN:=IncludeTrailingPathDelimiter(FDirectory)+E.Name;
  if FPortable then
    begin
    If FindFirst(FN,faAnyFile,PInfo)=0 then
      try
        UpdateDirectoryEntry(E,PInfo);
      finally
        FindClose(PInfo);
      end;
    end
  else
    begin
    If (fpStat(FN,UInfo)=0) then
      UpdateDirectoryEntry(E,UInfo);
    end;
end;

procedure TForm1.UpdateDirectoryEntry(E: TDirectoryEntry; const Info: TSearchRec);
begin
  // Convert available information.
  E.TimeStamp:=FileDateToDateTime(Info.Time);
  E.Attributes:=Info.Attr;
  E.Size:=Info.Size;
  E.Mode:=Info.Mode;
end;

procedure TForm1.UpdateDirectoryEntry(E : TDirectoryEntry; Const Info: Stat);

Var
  y,m,d,h,n,s,ms : word;
begin
  // Convert available information.
  epochtolocal(info.mtime,y,m,d,h,n,s);
  E.TimeStamp:=EncodeDate(y,m,d)+EncodeTime(h,n,s,info.mtime_nsec div 1000);
  E.Size:=Info.size;
  E.Mode:=Info.mode;
  E.Owner:=Info.uid;
  E.Group:=info.gid;
end;

procedure TForm1.MIPortableClick(Sender: TObject);
begin
  FPortable:=MIPortable.Checked;
  OpenDirectory(FDirectory);
end;

procedure TForm1.OpenDirectory(Const ADirectory : string);

begin
  if (ADirectory=FDirectory) then
    exit;
  FEntries.Clear;
  if FPortable then
    OpenPortableDirectory(ADirectory)
  else
    OpenUnixDirectory(ADirectory);
  FDirectory:=ADirectory;
  if (wd<>-1) then
    begin
    inotify_rm_watch(fd,wd);
    end;
  ShowDirectory;
  wd:=inotify_add_watch(fd,Pchar(FDirectory),NotifyEvents);
  Caption:='Showing directory '+FDirectory;
end;

procedure TForm1.UpdateItem (LI : TListItem);

Var
  E : TDirectoryEntry;

begin
  LVDir.BeginUpdate;
  try
    E:=TDirectoryEntry(Li.Data);
    LI.Caption:=E.Name;
    With LI.SubItems do
      begin;
      BeginUpdate;
      try
        Clear;
        Add(DateTimeToStr(E.TimeStamp));
        Add(FileSizeString(E.Size));
        Add(FilePermissionString(E.Mode));
        if FPortable then
          begin
          Add('?');
          Add('?');
          end
        else
          begin
          Add(FileOwnerString(E.Owner));
          Add(FileGroupString(E.Group));
          end;
      finally
        EndUpdate;
      end;
      end;
  finally
    LVDir.EndUpdate;
  end;
end;

function CompareDirs(Item1, Item2: TCollectionItem): Integer;

begin
  Result:=CompareText(TDirectoryEntry(Item1).Name,TDirectoryEntry(Item2).Name);
end;

procedure TForm1.ShowDirectory;

Var
  I : integer;
  Li : TListItem;

begin
  FEntries.Sort(@CompareDirs);

  With LVDir.Items do
    begin
    beginUpdate;
    try
      Clear;
      For I:=0 to FEntries.Count-1 do
        begin
        LI:=LVDir.Items.Add;
        LI.Data:=FEntries[i];
        UpdateItem(Li);
        end;
    finally
      EndUpdate;
    end;
    end;
end;

procedure TForm1.AddDirectoryEntry(Const Info : TSearchRec);

Var
  E : TDirectoryEntry;

begin
  E:=FEntries.AddEntry(Info.Name);
  UpdateDirectoryEntry(E,Info);
end;

procedure TForm1.OpenPortableDirectory(Const ADirectory : string);

Var
  Info : TSearchRec;

begin
  if FindFirst(IncludeTrailingPathDelimiter(ADirectory)+AllFilesMask,faAnyFile,Info)=0 then
    try
      Repeat
        AddDirectoryEntry(Info);
      Until FindNext(Info)<>0;
    finally
      FindClose(info);
    end;
end;

procedure TForm1.AddDirectoryEntry(Const AFileName : string; Info : Stat);

Var
  E : TDirectoryEntry;
begin
  E:=FEntries.AddEntry(AFileName);
  UpdateDirectoryEntry(E,Info);
end;

procedure TForm1.OpenUnixDirectory(Const ADirectory : string);

Var
  ADir : PDir;
  de : PDirEnt;
  dn,fn : string;
  Info : stat;
begin
  ADir:=fpOpenDir(ADirectory);
  dn:=IncludeTrailingPathDelimiter(ADirectory);
  if (ADir<>Nil) then
    try
      de:=fpReadDir(ADir^);
      While (de<>Nil) do
        begin
        fn:=de^.d_name;
        if fpStat(dn+FN,info)=0 then
          AddDirectoryEntry(FN,Info);
        de:=fpReadDir(ADir^);
        end;
    finally
      fpCloseDir(ADir^);
    end;
end;


end.

